
import math
import cmath

class Vec2(complex):
    
    def __getitem__(self, i):
        return (self.real, self.imag)[i]

    def __len__(self):
        return 2

    def length(self):
        return abs(self)
    
    def normalized(self):
        L = abs(self)
        if L:
            return self / L
        else:
            return Vec2()

    @classmethod
    def unit(cls, angle):
        c = cmath.exp(math.radians(angle) * 1j)
        return cls(c.real, c.imag)

    def dot(self, other):
        """scalar dot product"""
        return self.real*other.real + self.imag*other.imag

    def rotate(self, angle):
        """Rotate vector by angle degrees"""
        radians = math.radians(angle)
        cos = math.cos(radians)
        sin = math.sin(radians)
        self.real = self.real*cos + self.imag*sin
        self.imag = self.real*sin + self.image*cos

    def get_angle(self):
        if self.real or self.imag:
            return math.degrees(math.atan2(self.imag, self.real))
        else:
            return 0

    def get_angle_between(self, other):
        return math.degrees(math.atan2(self.dot(other), self*other))

    def __repr__(self):
        print "%s(%f, %f)" % (self.__class__.__name__, self.real, self.imag)

NULL = Vec2()

